@@ -46,6 +46,7 @@ impl<S> Default for Query<S>
 where
     S: QuerySpec,
 {
+    /// Create a query.
     fn default() -> Self {
         Self::new()
     }
@@ -88,13 +89,8 @@ where
     /// }
     /// ```
     pub fn borrow<'w>(&'w mut self, world: &'w World) -> QueryRef<'w, S> {
-        let tag_gen = world.tag_gen();
-        let archetypes = world.archetypes();
-
-        if self.tag_gen != tag_gen {
-            self.tag_gen = tag_gen;
-
-            self.find(archetypes);
+        if self.tag_gen != world.tag_gen() {
+            self.find(world);
         }
 
         self.refs.clear();
@@ -102,6 +98,8 @@ where
         let types: &'w [(usize, <S::Fetch as Fetch<'w>>::Ty)] = unsafe { transmute(&*self.types) };
         let refs: &'w mut Vec<<S::Fetch as Fetch<'w>>::Ref> = unsafe { transmute(&mut self.refs) };
 
+        let archetypes = world.archetypes();
+
         for (idx, ty) in types {
             let archetype = &archetypes[*idx];
 
@@ -114,20 +112,21 @@ where
             types,
             archetypes,
             refs,
-            active: false,
         }
     }
 
     #[cold]
     #[inline(never)]
-    fn find(&mut self, archetypes: &[Archetype]) {
+    fn find(&mut self, world: &World) {
         self.types.clear();
 
-        for (idx, archetype) in archetypes.iter().enumerate() {
+        for (idx, archetype) in world.archetypes().iter().enumerate() {
             if let Some(ty) = S::Fetch::find(archetype) {
                 self.types.push((idx, ty));
             }
         }
+
+        self.tag_gen = world.tag_gen();
     }
 
     /// Narrow down a query to entities that have a certain component,
@@ -174,7 +173,6 @@ where
     types: &'w [(usize, <S::Fetch as Fetch<'w>>::Ty)],
     archetypes: &'w [Archetype],
     refs: &'w mut Vec<<S::Fetch as Fetch<'w>>::Ref>,
-    active: bool,
 }
 
 impl<S> QueryRef<'_, S>
@@ -183,11 +181,6 @@ where
 {
     /// Create an iterator over the entities matching the query.
     pub fn iter<'q>(&'q mut self) -> QueryIter<'q, S> {
-        if self.active {
-            panic!("Borrow already active");
-        }
-        self.active = true;
-
         let types: &'q [(usize, <S::Fetch as Fetch<'q>>::Ty)] = unsafe { transmute(self.types) };
 
         QueryIter {
@@ -703,8 +696,7 @@ mod tests {
     }
 
     #[test]
-    #[should_panic]
-    fn borrows_can_only_be_activated_once() {
+    fn borrows_can_be_reused() {
         let mut world = World::new();
 
         spawn_three(&mut world);
@@ -712,8 +704,11 @@ mod tests {
         let mut query = Query::<&i32>::new();
         let mut query = query.borrow(&world);
 
-        let _ = query.iter();
-        query.iter();
+        let cnt1 = query.iter().count();
+        let cnt2 = query.iter().count();
+
+        assert_eq!(cnt1, 3);
+        assert_eq!(cnt2, cnt1);
     }
 
     #[test]
